/* ============================================================================
*******************************************************************************
************** U.S. Geological Survey preliminary computer program ************
*******************************************************************************
**************                      Infoutl.c                     *************
*******************************************************************************
===============================================================================
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
**
** Disclaimer: 
**          Although this program has been used by the U.S. Geological Survey,
**           no warranty, expressed or implied, is made by the USGS as to the
**           accuracy and functioning of the program and related program
**           material nor shall the fact of distribution constitute any such
**           warranty, and no responsibility is assumed by the USGS in
**           connection therewith.
**
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
===============================================================================

 Name: Read_Ubudsv.c

 Description: Loads values from a cell-by-cell term file output by the utility
               UBUDSV module of the USGS Modular Three-Dimensional 
               Finite-Difference Ground-Water Flow Model for version 88, 96,
               and 2000 into an array variable.

 Inputs: 
          OutputFilePath : Name of an ASCII text file to be created.
          DataFilePath : Name of a data file containing the values of the 
                          items.
          StressPeriod : Flag to identify stress period
          TimeStep     : Flag to identify time step within stress period

 Outputs: 
          arrayptr     : Cell-by-cell terms
          ierror       : error value (no error set to number of model layers)

 History:

     Author/Site,      Date,        Event
     ------------------------------------------------------------
   Leonard L. Orzol                                   Phone: (503) 251-3270
   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
   10615 SE. Cherry Blossom Drive
   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov

*/
/***************************************************************************/

#include <stdio.h>
#include <string.h>

/***************************************************************************/
struct ubudsv_info
{
       long int  current_time_step;
       long int  current_stress_period;
       char      current_label[16];
       long int  model_cols;
       long int  model_rows;
       long int  model_lays;
};
struct extra_info
{
       long int  format_type;
       float     delta_time;
       float     present_time;
       float     total_time;
};
/***************************************************************************/

void read_ubudsv__ ( 
                   DataFile,
                   Search_String,
                   StressPeriod,
                   TimeStep,
                   arrayptr,
                   ierror)

     char      *DataFile;
     char      *Search_String;
     long int  *StressPeriod;
     long int  *TimeStep;
     float     *arrayptr;
     long int  *ierror;

{
  char               Program_Name[256] = "Read_Ubudsv";
  float              version = 2.0;
  char               version_date[256] = "24May2002";

  struct ubudsv_info header;

  struct extra_info  extra_header;

  char               character;
  char              *HeaderText;
  char               Header[128];
  char               auxiliary_name[16];
  long int           number_of_layers = 0, n = 0;
  float             *dumbptr, qv;

  char               DataFilePath[256];
  FILE              *DataFilePtr = ( FILE * ) NULL;

  long int           i, ii, data_found = 0;
  long int           array_size, *integerptr, file_format = 1;
  long int           layers;
  long int           rows;
  long int           cols;
  long int           number_of_records = 0;
  long int           number_of_cells = 0;
  long int           size_of_file, save_pos;
  long int           ModelStressPeriod = (long int) *StressPeriod;
  long int           ModelTimeStep = (long int) *TimeStep;

  int                error = -9999, mode = SEEK_CUR;

  long int           nlay, nrow, ncol;
  long int           layer, row, column, rowcolumn, row_multiplier = 0;
  long int           nlist, nval, icrl, loc;

/* 
   Null terminate DataFilePath and Search_String
*/

 for( i = 0; i < 256; i++ )
    {
     character=*(DataFile+i);
     if( character==' ')
       {
        *(DataFilePath+i)='\0';
        break;
       }
     DataFilePath[i]=character;
    }

 *(Search_String+16)='\0';
 *ierror = error;

 fprintf(stdout,"\n     ..Searching file %s for text string !%s! \n  at time step %d in stress period %d",
          DataFilePath, Search_String, ModelTimeStep, ModelStressPeriod);

/* 
   Open Data File 
*/
 if(( DataFilePtr = fopen( DataFilePath, "rb" )) == NULL )
   {
    fprintf(stdout,"\nCan not open data file %s for reading\n",DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
    return;
   }

 save_pos = ftell (DataFilePtr);      
 fseek (DataFilePtr, 0L, SEEK_END);      
 size_of_file = ftell (DataFilePtr) - 1;      
 fseek (DataFilePtr, save_pos, SEEK_SET);

/* 
   Read Data File (end of file or necessary to ensure correct position)
*/

 while (!feof(DataFilePtr))
   {

    /* 
       Read Header Information in Data File 
    */
        
    if(fread((char *) &header,sizeof(struct ubudsv_info),1,DataFilePtr) != 1)
      {
       if(ferror(DataFilePtr))
         {
          fprintf(stdout,"\nCould not read header information");
          fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
          return;
         }
      }

    nrow = header.model_rows;
    ncol = header.model_cols;
    nlay = header.model_lays;
    fprintf(stdout,"\n\nHeader information for %s with layer set to %d",
                   header.current_label,nlay);

    /* 
       Identify format in Data File 
    */
    file_format = 1;
    if(nlay < 0) 
      {
       nlay = -1 * nlay;

       if(fread((char *) &extra_header,sizeof(struct extra_info),1,DataFilePtr) != 1)
         {
          if(ferror(DataFilePtr))
            {
             fprintf(stdout,"\nCould not read header information");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
            }
         }
        
       file_format = extra_header.format_type;

       fprintf(stdout,"\nFile format set to %d",file_format);
       fprintf(stdout,"\nSimulation time: %f",extra_header.present_time);
       fprintf(stdout,"\nTotal time: %f",extra_header.total_time);

      }

    array_size = nrow*ncol*nlay;

    /* 
       Check stress period and time step 
    */
    if(header.current_stress_period > ModelStressPeriod)
       break;
    if(header.current_stress_period >= ModelStressPeriod &&
       header.current_time_step > ModelTimeStep)
       break;
           
    fprintf(stdout,"\n %s \n",header.current_label);
         
    /* 
       Right data for right stress period and time step 
    */
    if((strncmp(header.current_label,Search_String,16) == 0) &&
       (ModelStressPeriod == header.current_stress_period) &&
       (ModelTimeStep == header.current_time_step))
      {
       *ierror = nlay;

       data_found = 1;
           
       /* 
          Read array (original unformatted format) 
       */
       if(file_format == 1)
         {
          if(fread(arrayptr,array_size*sizeof(float),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
	    }
         }

       /* 
          List without auxiliary values 
       */
       if(file_format == 2)
         {
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
           }
   
          fprintf(stdout,"\nFile format set to %d",file_format);
   
          fprintf(stdout,"\nNumber in list is %d",nlist);
   
          if(nlist <= 0) 
            {
             continue;
            }
   
          fprintf(stdout,"\n%10s %10s %10s %15s","Layer","Row","Column","Value");
          for(i = 0; i < nlist; i++)
            {
             /*
                Cell identifier
             */
             if(fread(&icrl,1 * sizeof(long int),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   return;
                  }
               }
   
             /*
                Cell value
             */
             if(fread(&qv,nval * sizeof(float),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   return;
                  }
               }
   
             /*
                Display cell value
             */
             rowcolumn = nrow*ncol;
             layer = ( icrl - 1 ) / rowcolumn + 1;
             row = (( icrl - ( layer - 1 ) * rowcolumn ) - 1)/ncol + 1;
             column = icrl - (layer - 1) * rowcolumn - (row - 1) * ncol;
             fprintf(stdout,"\n%10d %10d %10d %15f",layer,row,column,qv);

             loc = (layer - 1) * array_size + (row - 1) * ncol + column - 1;

             *(arrayptr + loc) = qv;
            }
         }

       /* 
          2-D array plus a layer indicator array 
       */
       if(file_format == 3)
         {

          array_size = nrow*ncol;
   
          /* 
             Allocate memory for arrays 
          */
          if(!(dumbptr = (float *) malloc(array_size*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
            }
   
          if(!(integerptr = (long int *) malloc(array_size*sizeof(long int *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
            }
   
          /* 
             Layer array 
          */
          if(fread(integerptr,array_size*sizeof(long int),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
	    }
   
          /* 
             Cell value array 
          */
          if(fread(dumbptr,array_size*sizeof(float),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
	    }
   
          /* 
             Loop to assign cell values array 
          */
          row = 1;
          column = 0;
          for(i = 0; i < array_size; i++)
            {
	     column++;
             if(column > ncol)
               {
		 row++;
                 column = 1;
               }
             layer = *(integerptr + i);

             loc = (layer - 1) * array_size + (row - 1) * ncol + column - 1;

             *(arrayptr + loc) = *(dumbptr + i);
            }
         }

       /* 
          2-D array for layer 1
       */
       if(file_format == 4)
         {

          array_size = nrow*ncol;
   
          /* 
             Allocate memory for arrays 
          */
          if(!(dumbptr = (float *) malloc(array_size*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
            }
   
          /* 
             Cell value array 
          */
          if(fread(arrayptr,array_size*sizeof(float),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
	    }
         }
    
       /* 
          List with auxiliary values 
       */
       if(file_format == 5)
         {
          fprintf(stdout,"\nList with auxiliary values");
   
          if(fread(&nval,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
           }
   
          fprintf(stdout,"\nNumber of auxiliaries is %d",nval);
   
          if(fread(auxiliary_name,16*sizeof(char),nval - 1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
           }
   
          fprintf(stdout,"\nAuxiliary name is %s",auxiliary_name);
   
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
            }
   
          fprintf(stdout,"\nNumber in list is %d",nlist);
   
          if(nlist <= 0) 
            {
             continue;
            }
   
          /* 
             Allocate memory for arrays 
          */
          if(!(dumbptr = (float *) malloc(nval*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             return;
            }
   
          /* 
             Loop for arrays 
          */
          fprintf(stdout,"\n%10s %10s %10s %10s %15s","Layer","Row","Column","Loc","Value");
          for(i = 0; i < nlist; i++)
            {
             /*
                Cell identifier
             */
             if(fread(&icrl,1 * sizeof(long int),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   return;
                  }
               }
            
             /*
                Cell value
             */
             if(fread(dumbptr,nval * sizeof(float),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   return;
                  }
               }
   
             /*
                Display cell value
             */
             rowcolumn = nrow*ncol;
             layer = ( icrl - 1 ) / rowcolumn + 1;
             row = (( icrl - ( layer - 1 ) * rowcolumn ) - 1)/ncol + 1;
             column = icrl - (layer - 1) * rowcolumn - (row - 1) * ncol;

             loc = (layer - 1) * nrow * ncol + (row - 1) * ncol + column - 1;

             *(arrayptr + loc) = *(dumbptr);

             fprintf(stdout,"\n%10d %10d %10d %10d",layer,row,column,loc);
             for(ii = 0; ii < nval; ii++)
                {
                 fprintf(stdout," %15.3f",*(dumbptr + ii));
                }
            }
         }
   
       fprintf(stdout,"\n");
           
       /* 
          Done
       */
       break;
      }
   
    /* 
       Wrong data (move byte pointer)
    */
    else
      {     
       /* 
          Read array (original unformatted format)           
       */
       if(file_format == 1)
         {
          error = fseek(DataFilePtr,array_size*sizeof(float),mode);
         }

       /* 
          List without auxiliary values 
       */
       if(file_format == 2)
         {
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
           }
   
          if(nlist <= 0) 
            {
             continue;
            }

          for(i = 0; i < nlist; i++)
            {
             error = fseek(DataFilePtr,1 * sizeof(long int),mode);
             error = fseek(DataFilePtr,1 * sizeof(float),mode);
            }
         }

       /* 
          2-D array plus a layer indicator array 
       */
       if(file_format == 3)
         {

          array_size = nrow*ncol;
   
          /* 
             Layer array 
          */
          error = fseek(DataFilePtr,array_size*sizeof(long int),mode);
   
          /* 
             Cell value array 
          */
          error = fseek(DataFilePtr,array_size*sizeof(float),mode);
         }

       /* 
          2-D array for layer 1
       */
       if(file_format == 4)
         {

          array_size = nrow*ncol;
   
          /* 
             Cell value array 
          */
          error = fseek(DataFilePtr,array_size*sizeof(float),mode);
         }
       
       /* 
          List with auxiliary values 
       */
       if(file_format == 5)
         {
   
          if(fread(&nval,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
           }
   
          fprintf(stdout,"\nNumber of auxiliaries is %d",nval);
   
          if(fread(auxiliary_name,16*sizeof(char),nval - 1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
           }
   
          fprintf(stdout,"\nAuxiliary name is %s",auxiliary_name);
   
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                return;
               }
            }
   
          fprintf(stdout,"\nNumber in list is %d",nlist);
   
          if(nlist <= 0) 
            {
             continue;
            }

          for(i = 0; i < nlist; i++)
            {
             error = fseek(DataFilePtr,1 * sizeof(long int),mode);
             error = fseek(DataFilePtr,nval * sizeof(float),mode);
            }
         }

       /* 
          Check for errors 
       */
       if(error > 0)
         {
          if(feof(DataFilePtr))
             fprintf(stdout,"\nPremature end of file text %d",error);
          else
             fprintf(stdout,"\nCould not read array %d",error);

          fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
          return;
         }
      }

    /* 
       Reached end of file (necessary to ensure correct position)
    */
    if(ftell(DataFilePtr) >= size_of_file)
       break;
   }
      
/* 
  Close Data File 
*/
 if(fclose(DataFilePtr) != 0)
   {
    fprintf(stdout,"\nUnable to close file %s\n", DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
    return;
   }

/* 
  Find any data 
*/
 if(data_found > 0)
   {
    fprintf(stdout,"\n\n     ..Found data for %s in file %s for %d layers\n", 
               Search_String,
               DataFilePath,
               nlay);
    *ierror = nlay;
   }
 else
   {
    fprintf(stdout,"\n\n     ..Unable to find data for %s in file %s\n", 
               Search_String,
               DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
    *ierror = nlay;
   }

 return;

}

/***************************************************************************/
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
===============================================================================

 Name: Read_Ulasav.c

 Description: Loads values from a cell-by-cell head term file output by the 
               utility ULASAV module of the USGS Modular Three-Dimensional 
               Finite-Difference Ground-Water Flow Model for version 88, 96,
               and 2000 into an array variable.

 Inputs: 
          OutputFilePath : Name of an ASCII text file to be created.
          DataFilePath : Name of a data file containing the values of the 
                          items.
          StressPeriod : Flag to identify stress period
          TimeStep     : Flag to identify time step within stress period

 Outputs: 
          arrayptr     : Cell-by-cell terms
          ierror       : error value (no error set to number of model layers)

 History:

     Author/Site,      Date,        Event
     ------------------------------------------------------------
   Leonard L. Orzol                                   Phone: (503) 251-3270
   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
   10615 SE. Cherry Blossom Drive
   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov

*/
/***************************************************************************/

#include <stdio.h>
#include <string.h>

/***************************************************************************/
struct ulasav_info
{
       long int  current_time_step;
       long int  current_stress_period;
       float     current_model_time;
       float     current_total_time;
       char      current_label[16];
       long int  model_cols;
       long int  model_rows;
       long int  model_lays;
};
/***************************************************************************/

void read_ulasav__ ( 
                   DataFile,
                   Search_String,
                   StressPeriod,
                   TimeStep,
                   arrayptr,
                   ierror)

     char      *DataFile;
     char      *Search_String;
     long int  *StressPeriod;
     long int  *TimeStep;
     float     *arrayptr;
     long int  *ierror;

{
  char               Program_Name[256] = "Read_Ulasav";
  float              version = 1.0;
  char               version_date[256] = "24May2002";

  struct ulasav_info header;

  char               character;
  char              *HeaderText;
  char               Header[128];
  long int           number_of_layers = 0, n = 0;
  float             *dumbptr;

  char               DataFilePath[256];
  FILE              *DataFilePtr = ( FILE * ) NULL;

  long int           i, ii;
  long int           array_size;
  long int           layer;
  long int           rows;
  long int           cols;
  long int           number_of_records = 0;
  long int           number_of_cells = 0;
  long int           size_of_file, save_pos;
  long int           ModelStressPeriod = (long int) *StressPeriod;
  long int           ModelTimeStep = (long int) *TimeStep;

  int                error = -9999, mode = SEEK_CUR;

  long int           maxlay = 0, nlay, nrow, ncol;
  long int           row, column, rowcolumn, row_multiplier;
  long int           record = 0;

/* 
   Null terminate DataFilePath and Search_String
*/

 for( i = 0; i < 128; i++ )
    {
     character=*(DataFile+i);
     if( character==' ')
       {
        *(DataFilePath+i)='\0';
        break;
       }
     DataFilePath[i]=character;
    }

 *(Search_String+16)='\0';
 *ierror = error;

 fprintf(stdout,"\n     ..Searching file %s for text string !%s! \n  at time step %d in stress period %d",
          DataFilePath, Search_String, ModelTimeStep, ModelStressPeriod);

/* 
   Open Data File 
*/
 if(( DataFilePtr = fopen( DataFilePath, "rb" )) == NULL )
   {
    fprintf(stdout,"\nCan not open data file %s for reading\n",DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
    return;
   }

 save_pos = ftell (DataFilePtr);      
 fseek (DataFilePtr, 0L, SEEK_END);      
 size_of_file = ftell (DataFilePtr) - 1;      
 fseek (DataFilePtr, save_pos, SEEK_SET);
 i = 0;

/* 
   Read Data File (end of file or necessary to ensure correct position)
*/

 while (!feof(DataFilePtr))
   {

    /* 
       Read Header Information in Data File 
    */
        
    if(fread((char *) &header,sizeof(struct ulasav_info),1,DataFilePtr) != 1)
      {
       if(ferror(DataFilePtr))
         {
          fprintf(stdout,"\nCould not read header information");
          fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
          return;
         }
      }
           
    nrow = header.model_rows;
    ncol = header.model_cols;
    nlay = header.model_lays;
    array_size = nrow*ncol;

    /* 
       Check stress period and time step 
    */
    if(header.current_stress_period > ModelStressPeriod)
       break;
    if(header.current_stress_period >= ModelStressPeriod &&
       header.current_time_step > ModelTimeStep)
       break;
         
    /* 
       Right data for right stress period and time step 
    */
    if((strncmp(header.current_label,Search_String,16) == 0) &&
       (ModelStressPeriod == header.current_stress_period) &&
       (ModelTimeStep == header.current_time_step))
      {
       *ierror = nlay;

       maxlay = nlay;
           
       /* 
          Read array 
       */
       if(fread(arrayptr+record,array_size*sizeof(float),1,DataFilePtr) != 1)
         {
          if(feof(DataFilePtr))
             fprintf(stdout,"\nPremature end of file text");
          else
             fprintf(stdout,"\nCould not read array");
          fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
          return;
         }
           
       /* 
          Move pointer
       */
       record += array_size;
      }
   
    /* 
       Wrong data 
    */
    else
      {     
       /* 
          Move byte cursor
       */
       error = fseek(DataFilePtr,array_size*sizeof(float),mode);

       /* 
          Check for errors 
       */
       if(error > 0)
         {
          if(feof(DataFilePtr))
             fprintf(stdout,"\nPremature end of file text %d",error);
          else
             fprintf(stdout,"\nCould not read array %d",error);

          fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
          return;
         }
      }

    /* 
       Reached end of file (necessary to ensure correct position)
    */
    if(ftell(DataFilePtr) >= size_of_file)
       break;
   }
      
/* 
  Close Data File 
*/
 if(fclose(DataFilePtr) != 0)
   {
    fprintf(stdout,"\nUnable to close file %s\n", DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
    return;
   }

/* 
  Find any data 
*/
 if(maxlay > 0)
   {
    fprintf(stdout,"\n\nFound data for %s in file %s for %d layers\n", 
               Search_String,
               DataFilePath,
               maxlay);
    *ierror = maxlay;
   }
 else
   {
    fprintf(stdout,"\n\nUnable to find data for %s in file %s\n", 
               Search_String,
               DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
    *ierror = -9999;
    return;
   }

 return;

}

/***************************************************************************/
