/* ============================================================================
*******************************************************************************
************** U.S. Geological Survey preliminary computer program ************
*******************************************************************************
**************                       Ubudsv.c                     *************
*******************************************************************************
===============================================================================
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
**
** Disclaimer: 
**          Although this program has been used by the U.S. Geological Survey,
**           no warranty, expressed or implied, is made by the USGS as to the
**           accuracy and functioning of the program and related program
**           material nor shall the fact of distribution constitute any such
**           warranty, and no responsibility is assumed by the USGS in
**           connection therewith.
**
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
===============================================================================

 Name: Ubudsv.c

 Description: Loads values from a cell-by-cell term file output by the utility
               UBUDSV module of the USGS Modular Three-Dimensional 
               Finite-Difference Ground-Water Flow Model for version 88, 96,
               and 200 into an array variable.

 Inputs: 
          OutputFilePath : Name of an ASCII text file to be created.
          DataFilePath : Name of a data file containing the values of the 
                          items.
          HeaderName   : Flags to identify header of package
                       sto --> cell-by-cell storage terms
                       frf --> cell-by-cell flow right face terms 
                       fff --> cell-by-cell flow front face terms 
                       flf --> cell-by-cell flow lower face terms 
                       con --> cell-by-cell constant-head terms 
                       riv --> cell-by-cell river terms 
                       rch --> cell-by-cell recharge terms 
                       wel --> cell-by-cell well terms 
                       drn --> cell-by-cell drain terms 
                       evt --> cell-by-cell evt terms 
                       ghb --> cell-by-cell general-head boundary terms 
                       str --> cell-by-cell stream leakage terms 
                       flo --> cell-by-cell streamflow terms 
                       ibs --> cell-by-cell aquifer compaction terms 
                       sub --> cell-by-cell aquifer subsidence terms 
                       maw --> cell-by-cell multi-aquifer terms 

          StressPeriod : Flag to identify stress period
          TimeStep     : Flag to identify time step within stress period

 Outputs: 
          Function ( int ) : Function status

 History:

     Author/Site,      Date,        Event
     ------------------------------------------------------------
   Leonard L. Orzol                                   Phone: (503) 251-3270
   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
   10615 SE. Cherry Blossom Drive
   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov

*/
/***************************************************************************/

#include <stdio.h>
#include <string.h>

/***************************************************************************/
struct header_info
{
       long int  current_time_step;
       long int  current_stress_period;
       char      current_label[16];
       long int  model_cols;
       long int  model_rows;
       long int  model_lays;
};
struct extra_info
{
       long int  format_type;
       float     delta_time;
       float     present_time;
       float     total_time;
};
/***************************************************************************/

void main( argc, argv)
 
int argc;
char **argv;

{
  char               Program_Name[256] = "Ulasav_Txt";
  float              version = 1.0;
  char               version_date[256] = "23October2002";

  struct header_info header;

  char              *OutputFilePath;
  char              *HeaderText;
  char               Header[128];
  long int           number_of_layers = 0, n = 0;
  float             *dumbptr;

  FILE              *DataFilePtr = ( FILE * ) NULL;
  FILE              *OutputFilePtr = ( FILE * ) NULL;

  long int           i, ii, data_found = 0;
  long int           array_size, ModflowLayer;
  long int           model_layer, layer;
  long int           rows;
  long int           cols;
  long int           number_of_items = 0;
  long int           number_of_records = 0;
  long int           number_of_cells = 0;
  long int           Allocate_flag = 0;
  long int           size_of_file, save_pos;

  int                error, mode = SEEK_CUR;

  long int           row, column, rowcolumn, row_multiplier;
  long int           record_number;

  double             TempDouble, DumbDouble;

  fprintf(stdout,"\nSearching file %s for text string %s \n  at time step %d in stress period %d",
          DataFilePath, search_string, StressPeriod, TimeStep);

/* Open Data File */
 
  if(( DataFilePtr = fopen( DataFilePath, "rb" )) == NULL )
    {
     fprintf(stdout,"\nCan not open data file %s for reading\n",DataFilePath);
     fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
     exit (9999);
    }

  save_pos = ftell (DataFilePtr);      
  fseek (DataFilePtr, 0L, SEEK_END);      
  size_of_file = ftell (DataFilePtr) - 1;      
  fseek (DataFilePtr, save_pos, SEEK_SET);
  fprintf(stdout,"\nFile size %d",size_of_file);

/* 
   Read Data File (end of file or necessary to ensure correct position)
*/

     while (!feof(DataFilePtr))
       {

        /* 
           Read Header Information in Data File 
        */
        
        if(fread((char *) &header,sizeof(struct header_info),1,DataFilePtr) != 1)
          {
           if(ferror(DataFilePtr))
             {
              fprintf(stdout,"\nCould not read header information");
              fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
              exit (9999);
             }
          }

        fprintf(stdout,"\n\n");
        fprintf(stdout,"\nCurrent text label |%16s|", header.current_label);
        fprintf(stdout,"\nCurrent time step %d", header.current_time_step);
        fprintf(stdout,"\nCurrent stress period %d", header.current_stress_period);
        fprintf(stdout,"\nNumber of columns in model %d", header.model_cols);
        fprintf(stdout,"\nNumber of rows in model %d", header.model_rows);
        fprintf(stdout,"\nCurrent layer in model %d\n", header.model_lays);
         
        /* 
           Allocate memory for arrays 
        */
        if(Allocate_flag == 0)
          {
	   Allocate_flag = 1;
           array_size = header.model_rows*header.model_cols*header.model_lays;
           fprintf(stdout,"\n\nAllocating %d\n",array_size);
   
           if(!(arrayptr = (float *) malloc(array_size*sizeof(float *))))
             {
              fprintf(stdout,"\nCould not allocate sufficient memory space for array");
              fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
              exit (9999);
             }
	  }

        /* 
           Check stress period and time step 
        */
        if(header.current_stress_period > StressPeriod)
           break;
        if(header.current_stress_period >= StressPeriod &&
           header.current_time_step > TimeStep)
           break;

         
        /* 
           Right data for right stress period and time step 
        */
        if((strncmp(header.current_label,search_string,16) == 0) &&
           (StressPeriod == header.current_stress_period) &&
           (TimeStep == header.current_time_step))
          {
	   n = ftell(DataFilePtr);  
           fprintf(stdout,"\nCurrent position %d",n);

           data_found = 1;
           number_of_layers = header.model_lays;
           fprintf(stdout,"\n\nReading \n");
           
           /* 
              Read array 
           */
           if(fread(arrayptr,array_size*sizeof(float),1,DataFilePtr) != 1)
             {
              if(feof(DataFilePtr))
                 fprintf(stdout,"\nPremature end of file text");
              else
                 fprintf(stdout,"\nCould not read array");
              fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
              exit (9999);
             }
           
           /* 
              Done
           */
           break;
	  }
   
        /* 
           Wrong data 
        */
        else
          {     
	   n = ftell(DataFilePtr);  
           fprintf(stdout,"\nCurrent position %d",n);

           /* 
              Move byte cursor
           */
           error = fseek(DataFilePtr,array_size*sizeof(float),mode);

           /* 
              Check for errors 
           */
           if(error > 0)
             {
              if(feof(DataFilePtr))
                 fprintf(stdout,"\nPremature end of file text %d",error);
              else
                 fprintf(stdout,"\nCould not read array %d",error);

              fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
              exit (9999);
             }
          }

        /* 
           Reached end of file (necessary to ensure correct position)
        */
        fprintf(stdout,"\nCurrent position %d",ftell(DataFilePtr));
        if(ftell(DataFilePtr) >= size_of_file)
	  break;
       }
      
/* Close Data File */

  if(fclose(DataFilePtr) != 0)
    {
     fprintf(stdout,"\nUnable to close file %s\n", DataFilePath);
     fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
     exit (9999);
    }

/* Find any data */

  if(data_found > 0)
    {
     fprintf(stdout,"\n\nFound data for %s in file %s for %d layers\n", 
               search_string,
               DataFilePath,
               header.model_lays);
    }
  else
    {
     fprintf(stdout,"\n\nUnable to find data for %s in file %s\n", 
               search_string,
               DataFilePath);
     fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
     exit (9999);
    }

/* Open Output File */
 
 if(( OutputFilePtr = fopen( OutputFilePath, "w+" )) == NULL )
   {
    fprintf(stdout,"\nCan't open output file %s for writing\n",OutputFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Program\n");
    exit (1);
   }

/* 
   Header 
*/
 fprintf(stdout,"\nWriting header information to output file %s\n",OutputFilePath);

/* Load values in ASCII text file */

 fprintf(stdout,"\nWriting cell values to output file %s\n",OutputFilePath);
  row=0;
  column=0;
  layer=1;
  for(i= 0; i <array_size; ++i)
     {
      number_of_cells++;
      number_of_records++;
      column++;
      if(column > header.model_cols)
        {
         row++;
         column=1;
        }
      if(number_of_records > header.model_rows*header.model_cols)
        {
         layer++;
         number_of_records=1;
        }
      TempDouble=*(arrayptr+i);
      fprintf(OutputFilePtr,"%15.6lf\n", TempDouble);
     }

 if(fclose(OutputFilePtr) != 0)
   fprintf(stdout,"\nUnable to close file %s\n", OutputFilePath);

/* Print messages */

  fprintf(stdout,"\nLoaded %d cells into text file %s", number_of_cells,
          OutputFilePath);
  fprintf(stdout,"\n for layer %d ", layer);
  fprintf(stdout,"\n for %d rows", header.model_rows);
  fprintf(stdout,"\n for %d columns", header.model_cols);
  fprintf(stdout,"\n for MODFLOW stress period %d",StressPeriod);
  fprintf(stdout,"\n for MODFLOW time period %d",TimeStep);
  fprintf(stdout,"\n");
  
  fprintf(stdout,"\nNormal Termination of Ubudsv_Subroutine\n");
  fprintf (stdout,"\n");

}

/***************************************************************************/
