/* ============================================================================
*******************************************************************************
************** U.S. Geological Survey preliminary computer program ************
*******************************************************************************
**************                     Residuals_txt.c                *************
*******************************************************************************
===============================================================================
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
**
** Disclaimer: 
**          Although this program has been used by the U.S. Geological Survey,
**           no warranty, expressed or implied, is made by the USGS as to the
**           accuracy and functioning of the program and related program
**           material nor shall the fact of distribution constitute any such
**           warranty, and no responsibility is assumed by the USGS in
**           connection therewith.
**
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
===============================================================================

 Name: Residuals_txt.c

 Description: Loads values from a LISTING file output by a utility
               module of the USGS Modular Three-Dimensional 
               Finite-Difference Ground-Water Flow Model for version
               2000 into a text file that has a comma-delimited format
               for Excel, ArcView, ArcGis tables.

 Inputs: 
          OutputFilePath : Name of an ASCII text file to be created.
          DataFilePath : Name of a data file containing the values of the 
                          items.

          StressPeriod : Flag to identify stress period
          TimeStep     : Flag to identify time step within stress period

 Outputs: 
          Function ( int ) : Function status

 History:

     Author/Site,      Date,        Event
     ------------------------------------------------------------
   Leonard L. Orzol                                   Phone: (503) 251-3270
   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
   10615 SE. Cherry Blossom Drive
   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov

*/
/***************************************************************************/

#include <stdio.h>
#include <string.h>

void Disclaimer();

/***************************************************************************/

int main( argc, argv)

int argc;
char **argv;

{
  char               Program_Name[256] = "Residuals_Txt";
  float              version = 1.0;
  char               version_date[256] = "04September2002";

  long int           obs_number;
  char               obs_label[13];
  float              measured_head;
  float              calculated_head;
  float              residuals;
  float              weight;
  float              weight_residuals;

  char              *OutputFilePath;
  char              *DataFilePath;
  char              *BufferPtr;
  char              *HeadPtr = "  OBS#    NAME          HEAD        HEAD      RESIDUAL";
  char              *FlowPtr = "   OBS#    NAME          FLOW       FLOW     RESIDUAL  WEIGHT**.5  RESIDUAL";
  char              *TimePtr = "          TIME SUMMARY AT END OF TIME STEP";
  char               Buffer[256], character[40];
  char              *PackageText;

  long int           stress_flag = 0, time_flag = 0;
  long int           StressPeriod = 1, stress_period;
  long int           TimeStep = 1, time_step;

  FILE              *DataFilePtr = ( FILE * ) NULL;
  FILE              *OutputFilePtr = ( FILE * ) NULL;

  long int           i, ii, num_obs, num_omits;
  long int           data_found = 0, nobs = 0, nomit = 0;
  long int           obs_flag = 0, obs_text = 0, res_text = 0;

/* Disclaimer */
 
  Disclaimer (Program_Name, version, version_date);

/* Parse command line */

  if( argc < 3 || argc > 5 )
    {
     fprintf( stderr,"\n\nUsage: residual_txt <Output File Name> <MODFLOW LIST File Name> {Stress Period} {Time Step}\n\n" );
     exit( 1 );
    }

  OutputFilePath=argv[1];
  DataFilePath=argv[2];

  if( argc >= 4 )
    {
     StressPeriod=strtol(argv[3],(char **)NULL,10);
     stress_flag = 1;
    }

  if( argc >= 5 )
    {
     TimeStep=strtol(argv[4],(char **)NULL,10);
     time_flag = 1;
    }

  fprintf(stdout,"\nSearching listing file %s \n  at time step %d for stress period %d",
          DataFilePath, StressPeriod, TimeStep);

/* Open Data File */
 
  if(( DataFilePtr = fopen( DataFilePath, "r" )) == NULL )
    {
     fprintf(stdout,"\nCan not open data file %s for reading\n",DataFilePath);
     fprintf(stdout,"\nAbnormal Termination of Residuals_Subroutine\n");
     exit (9999);
    }

/* 
   Header information
*/
  fprintf(stdout,"\n\n%40s","Package");
  fprintf(stdout," %-6s","Stress");
  fprintf(stdout," %-6s","Time");
  fprintf(stdout," %-8s","Number");
  fprintf(stdout," %-8s","Number");

  fprintf(stdout,"\n%40s","Name");
  fprintf(stdout," %-6s","Period");
  fprintf(stdout," %-6s","Step");
  fprintf(stdout," %-8s","Loaded");
  fprintf(stdout," %-8s","Omitted");

/* 
   Open Output File 
*/
 
 if(( OutputFilePtr = fopen( OutputFilePath, "w+" )) == NULL )
   {
    fprintf(stdout,"\nCan't open output file %s for writing\n",OutputFilePath);
    fprintf(stdout,"\nAbnormal Termination of Residuals_Program\n");
    exit (1);
   }
        
/* 
   Open Output File 
*/
 
fprintf(OutputFilePtr,"%s,","obs_number");
fprintf(OutputFilePtr,"%s,","obs_label");
fprintf(OutputFilePtr,"%s,","meas_head");
fprintf(OutputFilePtr,"%s,","calc_head");
fprintf(OutputFilePtr,"%s,","residuals");
fprintf(OutputFilePtr,"%s,","weight");
fprintf(OutputFilePtr,"%s,","weig_resid");
fprintf(OutputFilePtr,"%s,","stress_period");
fprintf(OutputFilePtr,"%s","time_step");
fprintf(OutputFilePtr,"\n");

/* 
   Read Data File (end of file or necessary to ensure correct position)
*/
 while (!feof(DataFilePtr))
   {

    /* 
       Read Lines in Listing File 
    */
    fgets (Buffer,256,DataFilePtr);

    if(feof(DataFilePtr))
      {
	break;
      }

    /* 
       Read Residual Lines in Listing File 
    */
    if(strncmp(Buffer,HeadPtr,strlen(HeadPtr)) == 0 ||
       strncmp(Buffer,FlowPtr,strlen(FlowPtr)) == 0)
      {
	obs_flag = 1;
        continue;
      }

    /* 
       Read Stress Period and Time Step in Listing File 
    */
    if(strncmp(Buffer,TimePtr,strlen(TimePtr)) == 0)
      {
        /* 
           Time Step
	*/
        BufferPtr = Buffer+42;
        sscanf (BufferPtr,"%d",&time_step);

        /* 
           Stress Period
	*/
        BufferPtr = BufferPtr+21;
        sscanf (BufferPtr,"%d",&stress_period);
      }

    /* 
       Read Package Name for Residual in Listing File 
    */
    if(strncmp(Buffer," DATA AT ",9) == 0)
      {
        num_obs = 0;
        num_omits = 0;
        
        ii = 0;
        for(i = 9; i < strlen(Buffer) - 1; i++)
	  {
            character[ii] = Buffer[i];
            ii++;
	  }
        fprintf(stdout,"\n%40s",character);
        fprintf(stdout,"%6d",stress_period);
        fprintf(stdout,"%6d",time_step);
      }
    if(strncmp(Buffer," DATA FOR FLOWS REPRESENTED USING THE ",38) == 0)
      {
        num_obs = 0;
        num_omits = 0;
        ii = 0;
        for(i = 38; i < strlen(Buffer) - 1; i++)
	  {
            character[ii] = Buffer[i];
            ii++;
	  }
        fprintf(stdout,"\n%40s",character);
        fprintf(stdout,"%6d",stress_period);
        fprintf(stdout,"%6d",time_step);
      }

    if(obs_flag == 1)
      {
	obs_flag = 2;
        continue;
      }

    if(obs_flag == 2 && strlen(Buffer) < 10)
      {
        fprintf(stdout," %8d",num_obs);
        fprintf(stdout," %8d",num_omits);
	obs_flag = 0;
        continue;
      }

    if(obs_flag == 2)
      {
        data_found = 1;
        BufferPtr = Buffer+36;
        
        if(strncmp(BufferPtr,"OMITTED",7) == 0)
	  {
	    nomit++;
            num_omits++;
	    continue;
	  }

        num_obs++;

        /* 
           Obs number
	*/
        BufferPtr = Buffer;
        sscanf (BufferPtr,"%d",&obs_number);

        /* 
           Obs label
	*/
        BufferPtr = BufferPtr+7;
        strncpy (obs_label,BufferPtr,11);
        strcat (obs_label,"\0");

        /* 
           Measured head
	*/
        BufferPtr = BufferPtr+12;
        sscanf (BufferPtr,"%f",&measured_head);

        /* 
           Calculated head
	*/
        BufferPtr = BufferPtr+12;
        sscanf (BufferPtr,"%f",&calculated_head);

        /* 
           Residuals
	*/
        BufferPtr = BufferPtr+12;
        sscanf (BufferPtr,"%f",&residuals);

        /* 
           Weight
	*/
        BufferPtr = BufferPtr+8;
        sscanf (BufferPtr,"%f",&weight);

        /* 
           Weight residuals
	*/
        BufferPtr = BufferPtr+16;
        sscanf (BufferPtr,"%f",&weight_residuals);
        
        /* 
           Record residuals
	*/
        if(stress_flag == 0 && time_flag == 0) 
          {
           nobs++;

           fprintf(OutputFilePtr,"%d,",obs_number);
           fprintf(OutputFilePtr,"%-12s,",obs_label);
           fprintf(OutputFilePtr,"%.3f,",measured_head);
           fprintf(OutputFilePtr,"%.3f,",calculated_head);
           fprintf(OutputFilePtr,"%.3f,",residuals);
           fprintf(OutputFilePtr,"%.3f,",weight);
           fprintf(OutputFilePtr,"%.3f,",weight_residuals);
           fprintf(OutputFilePtr,"%d,",stress_period);
           fprintf(OutputFilePtr,"%d",time_step);
           fprintf(OutputFilePtr,"%\n");
	  }
        if(stress_flag == 1 && time_flag == 0 && stress_period == StressPeriod) 
          {
           nobs++;

           fprintf(OutputFilePtr,"%d,",obs_number);
           fprintf(OutputFilePtr,"%-12s,",obs_label);
           fprintf(OutputFilePtr,"%.3f,",measured_head);
           fprintf(OutputFilePtr,"%.3f,",calculated_head);
           fprintf(OutputFilePtr,"%.3f,",residuals);
           fprintf(OutputFilePtr,"%.3f,",weight);
           fprintf(OutputFilePtr,"%.3f,",weight_residuals);
           fprintf(OutputFilePtr,"%d,",stress_period);
           fprintf(OutputFilePtr,"%d",time_step);
           fprintf(OutputFilePtr,"%\n");
	  }
        if(stress_flag == 1 && time_flag == 1 && stress_period == StressPeriod && time_step == TimeStep) 
          {
           nobs++;

           fprintf(OutputFilePtr,"%d,",obs_number);
           fprintf(OutputFilePtr,"%-12s,",obs_label);
           fprintf(OutputFilePtr,"%.3f,",measured_head);
           fprintf(OutputFilePtr,"%.3f,",calculated_head);
           fprintf(OutputFilePtr,"%.3f,",residuals);
           fprintf(OutputFilePtr,"%.3f,",weight);
           fprintf(OutputFilePtr,"%.3f,",weight_residuals);
           fprintf(OutputFilePtr,"%d,",stress_period);
           fprintf(OutputFilePtr,"%d",time_step);
           fprintf(OutputFilePtr,"%\n");
	  }
      }
   }
   
 fprintf(stdout,"\n");
      
/* 
  Close Data File 
*/
 if(fclose(DataFilePtr) != 0)
   {
    fprintf(stdout,"\nUnable to close file %s\n", DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Residuals_Subroutine\n");
    exit (9999);
   }

/* 
  Find any data 
*/
 if(data_found > 0)
   {
    fprintf(stdout,"\nFound residual data in file %s for %d observations\n", 
               DataFilePath,
               nobs);
   }
 else
   {
    fprintf(stdout,"\n\nUnable to find residual data in file %s\n", 
               DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Residuals_Subroutine\n");
    exit (9999);
   }

/* 
   Close Output File 
*/
 if(fclose(OutputFilePtr) != 0)
   fprintf(stdout,"\nUnable to close file %s\n", OutputFilePath);

/* 
   Print messages 
*/
 fprintf(stdout,"\n Residuals Results");
 fprintf(stdout,"\n ..read input file %s",DataFilePath);
 fprintf(stdout,"\n ..recorded %d records",nobs);
 fprintf(stdout,"\n ..in output file %s",OutputFilePath);
 fprintf(stdout,"\n ..found %d omitted records",nomit);
 if(stress_flag == 0 && time_flag == 0)
   fprintf(stdout,"\n ..for all MODFLOW stress periods and time steps");
 if(stress_flag == 1 && time_flag == 0)
   {
    fprintf(stdout,"\n ..for MODFLOW stress period %d",StressPeriod);
    fprintf(stdout,"\n ..for all MODFLOW time periods");
   }
 if(stress_flag == 1 && time_flag == 1)
   {
    fprintf(stdout,"\n ..for MODFLOW stress period %d",StressPeriod);
    fprintf(stdout,"\n ..for MODFLOW time period %d",TimeStep);
   }
 fprintf(stdout,"\n");
  
 fprintf(stdout,"\nNormal Termination of Residuals_Subroutine\n");
 fprintf (stdout,"\n");

}

/***************************************************************************/

void Disclaimer ( Program_Name, version, version_date )

char *Program_Name;
float version;
char *version_date;

/***************************************************************************/
/*
       Name: Disclaimer
    Purpose: Display USGS disclaimer text
*/
/***************************************************************************/
{
      char              fmtbuf[128];
      char              blank = ' ';
      short int         even_length;


  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n************** U.S. Geological Survey preliminary computer program ************");
  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n");
  even_length = 40 - ( short int ) ( strlen(Program_Name) / 2);
  sprintf( fmtbuf, "%%%dc", even_length );
  fprintf(stdout,fmtbuf,blank);
  fprintf(stdout,"%-s",Program_Name);
  fprintf(stdout,"\n                                   version %5.1f",version);
  fprintf(stdout,"\n                                   %14s                 ",version_date);
  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n===============================================================================");
  fprintf(stdout,"\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::");
  fprintf(stdout,"\n**");
  fprintf(stdout,"\n** Disclaimer: ");
  fprintf(stdout,"\n**          Although this program has been used by the U.S. Geological Survey,");
  fprintf(stdout,"\n**           no warranty, expressed or implied, is made by the USGS as to the");
  fprintf(stdout,"\n**           accuracy and functioning of the program and related program");
  fprintf(stdout,"\n**           material nor shall the fact of distribution constitute any such");
  fprintf(stdout,"\n**           warranty, and no responsibility is assumed by the USGS in");
  fprintf(stdout,"\n**           connection therewith.");
  fprintf(stdout,"\n**");
  fprintf(stdout,"\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::");
  fprintf(stdout,"\n===============================================================================");

      fprintf( stdout, "\n" );

}                                           /* end of Disclaimer         */
/*                                            end of Ubudsv.c           */
