/* ============================================================================
*******************************************************************************
************** U.S. Geological Survey preliminary computer program ************
*******************************************************************************
**************                       Ubudsv_txt.c                 *************
*******************************************************************************
===============================================================================
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
**
** Disclaimer: 
**          Although this program has been used by the U.S. Geological Survey,
**           no warranty, expressed or implied, is made by the USGS as to the
**           accuracy and functioning of the program and related program
**           material nor shall the fact of distribution constitute any such
**           warranty, and no responsibility is assumed by the USGS in
**           connection therewith.
**
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
===============================================================================

 Name: Ubudsv_txt.c

 Description: Loads values from a cell-by-cell term file output by the utility
               UBUDSV module of the USGS Modular Three-Dimensional 
               Finite-Difference Ground-Water Flow Model for version 88, 96,
               and 200 into an array variable.

 Inputs: 
          OutputFilePath : Name of an ASCII text file to be created.
          DataFilePath : Name of a data file containing the values of the 
                          items.
          HeaderName   : Flags to identify header of package
                       sto --> cell-by-cell storage terms
                       frf --> cell-by-cell flow right face terms 
                       fff --> cell-by-cell flow front face terms 
                       flf --> cell-by-cell flow lower face terms 
                       con --> cell-by-cell constant-head terms 
                       riv --> cell-by-cell river terms 
                       rch --> cell-by-cell recharge terms 
                       wel --> cell-by-cell well terms 
                       drn --> cell-by-cell drain terms 
                       evt --> cell-by-cell evt terms 
                       ghb --> cell-by-cell general-head boundary terms 
                       str --> cell-by-cell stream leakage terms 
                       flo --> cell-by-cell streamflow terms 
                       ibs --> cell-by-cell aquifer compaction terms 
                       sub --> cell-by-cell aquifer subsidence terms 
                       maw --> cell-by-cell multi-aquifer terms 

          StressPeriod : Flag to identify stress period
          TimeStep     : Flag to identify time step within stress period

 Outputs: 
          Function ( int ) : Function status

 History:

     Author/Site,      Date,        Event
     ------------------------------------------------------------
   Leonard L. Orzol                                   Phone: (503) 251-3270
   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
   10615 SE. Cherry Blossom Drive
   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov

*/
/***************************************************************************/

#include <stdio.h>
#include <string.h>

void Disclaimer();

/***************************************************************************/
struct ubudsv_info
{
       long int  current_time_step;
       long int  current_stress_period;
       char      current_label[16];
       long int  model_cols;
       long int  model_rows;
       long int  model_lays;
};
struct extra_info
{
       long int  format_type;
       float     delta_time;
       float     present_time;
       float     total_time;
};
/***************************************************************************/

int main( argc, argv)

int argc;
char **argv;

{
  char               Program_Name[256] = "Ubudsv_Txt";
  float              version = 2.0;
  char               version_date[256] = "22May2002";

  struct ubudsv_info header;

  struct extra_info  extra_header;

  char              *OutputFilePath;
  char              *DataFilePath;
  char              *HeaderText;
  char               Search_String[17];
  char               rowtext[17], columntext[17];
  char               auxiliary_name[16];

  long int           StressPeriod = 1;
  long int           TimeStep = 1;
  float             *arrayptr, *dumbptr, qv;

  FILE              *DataFilePtr = ( FILE * ) NULL;
  FILE              *OutputFilePtr = ( FILE * ) NULL;

  long int           i, ii, iii, data_found = 0, n = 0;
  long int           array_size, *integerptr, file_format = 1;
  long int           number_of_records = 0;
  long int           size_of_file, save_pos;

  int                error = 0, mode = SEEK_CUR;

  long int           nlay, nrow, ncol;
  long int           layer, rowcolumn, seqnum;
  long int           row, row_multiplier;
  long int           column, column_multiplier;
  long int           nlist, nval, icrl, loc;

/* Disclaimer */
 
  Disclaimer (Program_Name, version, version_date);

/* Parse command line */

  if( argc < 4 || argc > 6 )
    {
     fprintf( stderr,"\n\nUsage: ubudsv <Output File Name> <MODFLOW Binary File Name> <Header Text: sto | frf | fff | flf | con | riv | rch | wel | drn | evt | ghb | str | flo | ibs | sub | com | cri | maw> {Stress Period} {Time Step}\n\n" );
     exit( 1 );
    }

  OutputFilePath=argv[1];
  DataFilePath=argv[2];
  HeaderText=argv[3];

  if( argc >= 5 )
    StressPeriod=strtol(argv[4],(char **)NULL,10);

  if( argc >= 6 )
    TimeStep=strtol(argv[5],(char **)NULL,10);

/* Check Header String */

  if(strncmp(HeaderText,"sto",3) == 0)
     strncpy(Search_String,"         STORAGE\0",17);
  else if(strncmp(HeaderText,"frf",3) == 0)
     strncpy(Search_String,"FLOW RIGHT FACE \0",17);
  else if(strncmp(HeaderText,"fff",3) == 0)
     strncpy(Search_String,"FLOW FRONT FACE \0",17);
  else if(strncmp(HeaderText,"flf",3) == 0)
     strncpy(Search_String,"FLOW LOWER FACE \0",17);
  else if(strncmp(HeaderText,"con",3) == 0)
     strncpy(Search_String,"   CONSTANT HEAD\0",17);
  else if(strncmp(HeaderText,"riv",3) == 0)
     strncpy(Search_String,"   RIVER LEAKAGE\0",17);
  else if(strncmp(HeaderText,"rch",3) == 0)
     strncpy(Search_String,"        RECHARGE\0",17);
  else if(strncmp(HeaderText,"wel",3) == 0)
     strncpy(Search_String,"           WELLS\0",17);
  else if(strncmp(HeaderText,"drn",3) == 0)
     strncpy(Search_String,"          DRAINS\0",17);
  else if(strncmp(HeaderText,"evt",3) == 0)
     strncpy(Search_String,"              ET\0",17);
  else if(strncmp(HeaderText,"ghb",3) == 0)
     strncpy(Search_String," HEAD DEP BOUNDS\0",17);
  else if(strncmp(HeaderText,"str",3) == 0)
     strncpy(Search_String,"  STREAM LEAKAGE\0",17);
  else if(strncmp(HeaderText,"flo",3) == 0)
     strncpy(Search_String,"  STREAM FLOWOUT\0",17);
  else if(strncmp(HeaderText,"ibs",3) == 0)
     strncpy(Search_String,"INTERBED STORAGE\0",17);
  else if(strncmp(HeaderText,"sub",3) == 0)
     strncpy(Search_String,"      SUBSIDENCE\0",17);
  else if(strncmp(HeaderText,"com",3) == 0)
     strncpy(Search_String,"      COMPACTION\0",17);
  else if(strncmp(HeaderText,"cri",3) == 0)
     strncpy(Search_String,"   CRITICAL HEAD\0",17);
  else if(strncmp(HeaderText,"maw",3) == 0)
     strncpy(Search_String,"MULTI-AQIFR WELL\0",17);
  else 
    {
     fprintf(stdout,"\nInvalid text string %s; ",HeaderText);
     fprintf(stdout,"\n  Package          Cell-by-Cell Term      Use");
     fprintf(stdout,"\n  Bcf1,Bcf2,Bcf3             storage      sto");
     fprintf(stdout,"\n  Bcf1,Bcf2,Bcf3     flow right face      frf");
     fprintf(stdout,"\n  Bcf1,Bcf2,Bcf3     flow front face      fff");
     fprintf(stdout,"\n  Bcf1,Bcf2,Bcf3     flow lower face      flf");
     fprintf(stdout,"\n  Riv1                 river leakage      riv");
     fprintf(stdout,"\n  Rch1                      recharge      rch");
     fprintf(stdout,"\n  Wel1                         wells      wel");
     fprintf(stdout,"\n  Drn1                        drains      drn");
     fprintf(stdout,"\n  Evt1                            et      evt");
     fprintf(stdout,"\n  Gdb1               head dep bounds      ghb");
     fprintf(stdout,"\n  Str1                stream leakage      str");
     fprintf(stdout,"\n  Str1                stream flowout      flo");
     fprintf(stdout,"\n  Ibs1              interbed storage      ibs");
     fprintf(stdout,"\n  Ibs1                    subsidence      sub");
     fprintf(stdout,"\n  Ibs1                    compaction      com");
     fprintf(stdout,"\n  Ibs1                 critical head      cri");
     fprintf(stdout,"\n  Maw1            multi-aquifer well      maw");
     fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
     exit (9999);
    }

  fprintf(stdout,"\nSearching file %s for text string %s \n  at time step %d in stress period %d",
          DataFilePath, Search_String, StressPeriod, TimeStep);

/* Open Data File */
 
  if(( DataFilePtr = fopen( DataFilePath, "rb" )) == NULL )
    {
     fprintf(stdout,"\nCan not open data file %s for reading\n",DataFilePath);
     fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
     exit (9999);
    }

  save_pos = ftell (DataFilePtr);      
  fseek (DataFilePtr, 0L, SEEK_END);      
  size_of_file = ftell (DataFilePtr) - 1;      
  fseek (DataFilePtr, save_pos, SEEK_SET);
  fprintf(stdout,"\nFile size %d",size_of_file);

/* 
   Header information
*/
  fprintf(stdout,"\n\n%20s","Text");
  fprintf(stdout," %6s","Stress");
  fprintf(stdout," %6s","Time");
  fprintf(stdout," %6s","Model");
  fprintf(stdout," %7s","Model");
  fprintf(stdout," %6s","Model");
  fprintf(stdout," %6s","File");
  fprintf(stdout," %6s","Aux");
  fprintf(stdout," %16s","Aux");
  fprintf(stdout," %6s","Number");

  fprintf(stdout,"\n%20s","header");
  fprintf(stdout," %6s","period");
  fprintf(stdout," %6s","step");
  fprintf(stdout," %6s","Rows");
  fprintf(stdout," %7s","Columns");
  fprintf(stdout," %6s","Layers");
  fprintf(stdout," %6s","format");
  fprintf(stdout," %6s","terms");
  fprintf(stdout," %16s","Name");
  fprintf(stdout," %6s","in list");

/* 
   Open Output File 
*/
 
 if(( OutputFilePtr = fopen( OutputFilePath, "w+" )) == NULL )
   {
    fprintf(stdout,"\nCan't open output file %s for writing\n",OutputFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Program\n");
    exit (1);
   }

/* 
   Read Data File (end of file or necessary to ensure correct position)
*/
 while (!feof(DataFilePtr))
   {
             
    fprintf(stdout,"\n Pointer position %d\n",ftell(DataFilePtr));

    /* 
       Read Header Information in Data File 
    */
    if(fread(&header,sizeof(struct ubudsv_info),1,DataFilePtr) != 1)
      {
       if(ferror(DataFilePtr))
         {
          fprintf(stdout,"\nCould not read header information");
          fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
          exit (9999);
         }
      }

    nrow = header.model_rows;
    sprintf(rowtext,"%d",nrow);
    sprintf(rowtext,"%d",strlen(rowtext));
    row_multiplier = 1;
    for(i = 0; i < atoi(rowtext); i++)
      {
	row_multiplier *= 10;
      }
    ncol = header.model_cols;
    sprintf(columntext,"%d",ncol);
    sprintf(columntext,"%d",strlen(columntext));
    column_multiplier = 1;
    for(i = 0; i < atoi(columntext); i++)
      {
	column_multiplier *= 10;
      }
    nlay = header.model_lays;
    for(i = 0; i < strlen(header.current_label); i++)
      {
	if(header.current_label[i] == 'l')
          header.current_label[i] = '\0';
      }
    fprintf(stdout,"\n%20s",header.current_label);
    fprintf(stdout," %6d",header.current_stress_period);
    fprintf(stdout," %6d",header.current_time_step);
    fprintf(stdout," %6d",nrow);
    fprintf(stdout," %7d",ncol);
    fprintf(stdout," %6d",nlay);

    /* 
       Identify format in Data File 
    */
    file_format = 1;
    if(nlay < 0) 
      {
       nlay = -1 * nlay;

       if(fread((char *) &extra_header,sizeof(struct extra_info),1,DataFilePtr) != 1)
         {
          if(ferror(DataFilePtr))
            {
             fprintf(stdout,"\nCould not read header information");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
            }
         }
        
       file_format = extra_header.format_type;

      }

    fprintf(stdout," %6d",file_format);

    array_size = nrow*ncol*nlay;

    /* 
       Check stress period and time step 
    */
    if(header.current_stress_period > StressPeriod)
       break;
    if(header.current_stress_period >= StressPeriod &&
       header.current_time_step > TimeStep)
       break;
           
    /* 
       Right data for right stress period and time step 
    */
    if((strncmp(header.current_label,Search_String,16) == 0) &&
       (StressPeriod == header.current_stress_period) &&
       (TimeStep == header.current_time_step))
      {

       data_found = 1;
           
       /* 
          Read array (original unformatted format) 
       */
       if(file_format == 1)
         {

          array_size = nrow*ncol*nlay;

          /* 
             Allocate memory for array 
          */
          if(!(arrayptr = (float *) malloc(nrow*ncol*nlay*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Grid_Subroutine\n");
             exit (9999);
            }
                
          if(fread(arrayptr,array_size*sizeof(float),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
	    }
   
          /* 
             Field header
          */
          fprintf(OutputFilePtr,"%s,%s,%s,%s,%s,%s","Seqnum","Layer","Row","Column","Rowcolumn","Value");
   
          /* 
             Loop to assign cell values array 
          */
          number_of_records = array_size;
          row = 1;
          column = 0;
          for(i = 0; i < array_size; i++)
            {
	     column++;
             if(column > ncol)
               {
		 row++;
                 column = 1;
               }
             layer = 1;
             seqnum = ( layer - 1 ) * rowcolumn + (row - 1) * ncol + column;
             rowcolumn = row * column_multiplier + column;

             fprintf(OutputFilePtr,"\n%10d,%10d,%10d,%10d,%10d,%15.3f",
                     seqnum,layer,row,column,rowcolumn,*(arrayptr + i));
            }
         }

       /* 
          List without auxiliary values 
       */
       if(file_format == 2)
         {
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
           }
   
          if(nlist <= 0) 
            {
             continue;
            }
   
          /* 
             Field header
          */
          fprintf(OutputFilePtr,"%s,%s,%s,%s,%s,%s","Seqnum","Layer","Row","Column","Rowcolumn","Value");

          number_of_records = nlist;
          for(i = 0; i < nlist; i++)
            {
             /*
                Cell identifier
             */
             if(fread(&icrl,1 * sizeof(long int),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   exit (9999);
                  }
               }
   
             /*
                Cell value
             */
             if(fread(&qv,nval * sizeof(float),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   exit (9999);
                  }
               }
   
             /*
                Display cell value
             */
             rowcolumn = nrow*ncol;
             layer = ( icrl - 1 ) / rowcolumn + 1;
             row = (( icrl - ( layer - 1 ) * rowcolumn ) - 1)/ncol + 1;
             column = icrl - (layer - 1) * rowcolumn - (row - 1) * ncol;
             fprintf(OutputFilePtr,"\n%10d,%10d,%10d,%10d,%10d,%15.3f",
                     seqnum,layer,row,column,rowcolumn,qv);
            }
         }

       /* 
          2-D array plus a layer indicator array 
       */
       if(file_format == 3)
         {

          array_size = nrow*ncol;
   
          /* 
             Allocate memory for arrays 
          */
          if(!(arrayptr = (float *) malloc(array_size*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
            }
   
          if(!(integerptr = (long int *) malloc(array_size*sizeof(long int *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
            }
   
          /* 
             Layer array 
          */
          if(fread(integerptr,array_size*sizeof(long int),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
	    }
   
          /* 
             Cell value array 
          */
          if(fread(arrayptr,array_size*sizeof(float),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
	    }
   
          /* 
             Field header
          */
          fprintf(OutputFilePtr,"%s,%s,%s,%s,%s,%s","Seqnum","Layer","Row","Column","Rowcolumn","Value");
   
          /* 
             Loop to assign cell values array 
          */
          number_of_records = array_size;
          row = 1;
          column = 0;
          for(i = 0; i < array_size; i++)
            {
	     column++;
             if(column > ncol)
               {
		 row++;
                 column = 1;
               }
             layer = *(integerptr + i);

             seqnum = ( layer - 1 ) * nrow * ncol + (row - 1) * ncol + column;
             rowcolumn = row * column_multiplier + column;

             fprintf(OutputFilePtr,"\n%10d,%10d,%10d,%10d,%10d,%15.3f",
                     seqnum,layer,row,column,rowcolumn,*(arrayptr + i));
            }
         }

       /* 
          2-D array for layer 1
       */
       if(file_format == 4)
         {

          array_size = nrow*ncol;
   
          /* 
             Allocate memory for arrays 
          */
          if(!(arrayptr = (float *) malloc(array_size*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
            }
   
          /* 
             Cell value array 
          */
          if(fread(arrayptr,array_size*sizeof(float),1,DataFilePtr) != 1)
            {
             if(feof(DataFilePtr))
                fprintf(stdout,"\nPremature end of file text");
             else
                fprintf(stdout,"\nCould not read array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
	    }
   
          /* 
             Field header
          */
          fprintf(OutputFilePtr,"%s,%s,%s,%s,%s,%s","Seqnum","Layer","Row","Column","Rowcolumn","Value");
   
          /* 
             Loop to assign cell values array 
          */
          number_of_records = array_size;
          row = 1;
          column = 0;
          for(i = 0; i < array_size; i++)
            {
	     column++;
             if(column > ncol)
               {
		 row++;
                 column = 1;
               }
             layer = 1;
             seqnum = ( layer - 1 ) * rowcolumn + (row - 1) * ncol + column;
             rowcolumn = row * column_multiplier + column;

             fprintf(OutputFilePtr,"\n%10d,%10d,%10d,%10d,%10d,%15.3f",
                     seqnum,layer,row,column,rowcolumn,*(arrayptr + i));
            }

         }

       /* 
          List with auxiliary values 
       */
       if(file_format == 5)
         {
          if(fread(&nval,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
           }
   
          fprintf(stdout," %6d",nval - 1);
   
          if(fread(auxiliary_name,16*sizeof(char),nval - 1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
           }
   
          fprintf(stdout," %16s",auxiliary_name);
   
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
            }
   
          fprintf(stdout," %6d",nlist);
   
          if(nlist <= 0) 
            {
             continue;
            }
   
          /* 
             Allocate memory for arrays 
          */
          if(!(arrayptr = (float *) malloc(nval*sizeof(float *))))
            {
             fprintf(stdout,"\nCould not allocate sufficient memory space for array");
             fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
             exit (9999);
            }
   
          /* 
             Field header
          */
          fprintf(OutputFilePtr,"%s,%s,%s,%s,%s,%s","Seqnum","Layer","Row","Column","Rowcolumn","Value");
          for(i = 1; i < nval; i++)
            {
             fprintf(OutputFilePtr,",");
             for(ii = 15; ii > 0; ii--)
	       {
	        if(auxiliary_name[ii] != ' ')
		  break;
	       }
             for(iii = 0; iii <= ii; iii++)
                {
                 fprintf(OutputFilePtr,"%c",auxiliary_name[iii]);             
                }
            }
   
          /* 
             Loop for arrays 
          */
          number_of_records = nlist;
          for(i = 0; i < nlist; i++)
            {
             /*
                Cell identifier
             */
             if(fread(&icrl,1 * sizeof(long int),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   exit (9999);
                  }
               }
            
             /*
                Cell value
             */
             if(fread(arrayptr,nval * sizeof(float),1,DataFilePtr) != 1)
               {
                if(ferror(DataFilePtr))
                  {
                   fprintf(stdout,"\nCould not read list information");
                   fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                   exit (9999);
                  }
               }
   
             /*
                Display cell value
             */
             rowcolumn = nrow*ncol;
             layer = ( icrl - 1 ) / rowcolumn + 1;
             row = (( icrl - ( layer - 1 ) * rowcolumn ) - 1)/ncol + 1;
             column = icrl - (layer - 1) * rowcolumn - (row - 1) * ncol;
             seqnum = ( layer - 1 ) * rowcolumn + (row - 1) * ncol + column;
             seqnum = rowcolumn + (row - 1) * ncol + column;
             rowcolumn = row * column_multiplier + column;

             fprintf(OutputFilePtr,"\n%d,%d,%d,%d,%d",seqnum,layer,row,column,rowcolumn);
             for(ii = 0; ii < nval; ii++)
                {
                 fprintf(OutputFilePtr,",%.3f",*(arrayptr + ii));
                }
            }
         }
   
       fprintf(stdout,"\n");
             
       fprintf(stdout,"\n Pointer position %d",ftell(DataFilePtr));
       fprintf(stdout,"  Array size %d \n",array_size);
           
       /* 
          Done
       */
       break;
      }
   
    /* 
       Wrong data (move byte pointer)
    */
    else
      {     
 
       /* 
          Read array (original unformatted format)           
       */
       if(file_format == 1)
         {
          error = fseek(DataFilePtr,array_size*sizeof(float),mode);
         }

       /* 
          List without auxiliary values 
       */
       if(file_format == 2)
         {
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
           }
   
          if(nlist <= 0) 
            {
             continue;
            }

          for(i = 0; i < nlist; i++)
            {
             error = fseek(DataFilePtr,1 * sizeof(long int),mode);
             error = fseek(DataFilePtr,1 * sizeof(float),mode);
            }
         }

       /* 
          2-D array plus a layer indicator array 
       */
       if(file_format == 3)
         {

          array_size = nrow*ncol;
   
          /* 
             Layer array 
          */
          error = fseek(DataFilePtr,array_size*sizeof(long int),mode);
   
          /* 
             Cell value array 
          */
          error = fseek(DataFilePtr,array_size*sizeof(float),mode);
         }

       /* 
          2-D array for layer 1
       */
       if(file_format == 4)
         {

          array_size = nrow*ncol;
   
          /* 
             Cell value array 
          */
          error = fseek(DataFilePtr,array_size*sizeof(float),mode);
         }
       
       /* 
          List with auxiliary values 
       */
       if(file_format == 5)
         {
          if(fread(&nval,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
           }
   
          fprintf(stdout," %6d",nval - 1);
   
          if(fread(auxiliary_name,16*sizeof(char),nval - 1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
           }
   
          fprintf(stdout," %-16s",auxiliary_name);
   
          if(fread(&nlist,1 * sizeof(long int),1,DataFilePtr) != 1)
            {
             if(ferror(DataFilePtr))
               {
                fprintf(stdout,"\nCould not read list information");
                fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
                exit (9999);
               }
            }
   
          fprintf(stdout," %6d",nlist);
   
          if(nlist <= 0) 
            {
             continue;
            }
   
          /* 
             Loop over arrays 
          */
          for(i = 0; i < nlist; i++)
            {
             error = fseek(DataFilePtr,1 * sizeof(long int),mode);
             error = fseek(DataFilePtr,nval * sizeof(float),mode);
            }
         }

       /* 
          Check for errors 
       */
       if(error > 0)
         {
          if(feof(DataFilePtr))
             fprintf(stdout,"\nPremature end of file text %d",error);
          else
             fprintf(stdout,"\nError %d",error);

          fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
          exit (9999);
         }
      }

    /* 
       Reached end of file (necessary to ensure correct position)
    */
    if(ftell(DataFilePtr) >= size_of_file)
       break;
    if(feof(DataFilePtr))
       break;
             
    fprintf(stdout,"\n Pointer position %d",ftell(DataFilePtr));
    fprintf(stdout,"  Array size %d \n",array_size);

   }
      
/* 
  Close Data File 
*/
 if(fclose(DataFilePtr) != 0)
   {
    fprintf(stdout,"\nUnable to close file %s\n", DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
    exit (9999);
   }

/* 
  Find any data 
*/
 if(data_found > 0)
   {
    fprintf(stdout,"\n\n     ..Found data for %s in file %s for %d layers\n", 
               Search_String,
               DataFilePath,
               nlay);
   }
 else
   {
    fprintf(stdout,"\n\nUnable to find data for %s in file %s\n", 
               Search_String,
               DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ubudsv_Subroutine\n");
    exit (9999);
   }

/* 
   Close Output File 
*/
 if(fclose(OutputFilePtr) != 0)
   fprintf(stdout,"\nUnable to close file %s\n", OutputFilePath);

/* 
   Print messages 
*/
 fprintf(stdout,"\nLoaded %d records into text file %s", 
                number_of_records,OutputFilePath);
 fprintf(stdout,"\n for layer %d ", nlay);
 fprintf(stdout,"\n for %d rows", nrow);
 fprintf(stdout,"\n for %d columns", ncol);
 fprintf(stdout,"\n for MODFLOW stress period %d",StressPeriod);
 fprintf(stdout,"\n for MODFLOW time period %d",TimeStep);
 fprintf(stdout,"\n");
  
 fprintf(stdout,"\nNormal Termination of Ubudsv_Subroutine\n");
 fprintf (stdout,"\n");

}

/***************************************************************************/

void Disclaimer ( Program_Name, version, version_date )

char *Program_Name;
float version;
char *version_date;

/***************************************************************************/
/*
       Name: Disclaimer
    Purpose: Display USGS disclaimer text
*/
/***************************************************************************/
{
      char              fmtbuf[128];
      char              blank = ' ';
      short int         even_length;


  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n************** U.S. Geological Survey preliminary computer program ************");
  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n");
  even_length = 40 - ( short int ) ( strlen(Program_Name) / 2);
  sprintf( fmtbuf, "%%%dc", even_length );
  fprintf(stdout,fmtbuf,blank);
  fprintf(stdout,"%-s",Program_Name);
  fprintf(stdout,"\n                                   version %5.1f",version);
  fprintf(stdout,"\n                                   %14s                 ",version_date);
  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n===============================================================================");
  fprintf(stdout,"\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::");
  fprintf(stdout,"\n**");
  fprintf(stdout,"\n** Disclaimer: ");
  fprintf(stdout,"\n**          Although this program has been used by the U.S. Geological Survey,");
  fprintf(stdout,"\n**           no warranty, expressed or implied, is made by the USGS as to the");
  fprintf(stdout,"\n**           accuracy and functioning of the program and related program");
  fprintf(stdout,"\n**           material nor shall the fact of distribution constitute any such");
  fprintf(stdout,"\n**           warranty, and no responsibility is assumed by the USGS in");
  fprintf(stdout,"\n**           connection therewith.");
  fprintf(stdout,"\n**");
  fprintf(stdout,"\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::");
  fprintf(stdout,"\n===============================================================================");

      fprintf( stdout, "\n" );

}                                           /* end of Disclaimer         */
/*                                            end of Ubudsv.c           */
