/* ============================================================================
*******************************************************************************
************** U.S. Geological Survey preliminary computer program ************
*******************************************************************************
**************                       Ulasav.c                     *************
*******************************************************************************
===============================================================================
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
**
** Disclaimer: 
**          Although this program has been used by the U.S. Geological Survey,
**           no warranty, expressed or implied, is made by the USGS as to the
**           accuracy and functioning of the program and related program
**           material nor shall the fact of distribution constitute any such
**           warranty, and no responsibility is assumed by the USGS in
**           connection therewith.
**
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
===============================================================================

 Name: Ubudsv.c

 Description: Loads values from a cell-by-cell term file output by the utility
               ULASAV module of the USGS Modular Three-Dimensional 
               Finite-Difference Ground-Water Flow Model for version 88, 96,
               and 2000 into an array variable.

 Inputs: 
          OutputFilePath : Name of an ASCII text file to be created.
          DataFilePath : Name of a data file containing the values of the 
                          items.
          HeaderName   : Flags to identify header of package
                       hed --> cell-by-cell head terms
                       ddn --> cell-by-cell drawdown terms 

          StressPeriod : Flag to identify stress period
          TimeStep     : Flag to identify time step within stress period

 Outputs: 
          Function ( int ) : Function status

 History:

     Author/Site,      Date,        Event
     ------------------------------------------------------------
   Leonard L. Orzol                                   Phone: (503) 251-3270
   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
   10615 SE. Cherry Blossom Drive
   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov

*/
/***************************************************************************/

#include <stdio.h>
#include <string.h>

/***************************************************************************/
struct ulasav_info
{
       long int  current_time_step;
       long int  current_stress_period;
       float     current_model_time;
       float     current_total_time;
       char      current_label[16];
       long int  model_cols;
       long int  model_rows;
       long int  model_lays;
};
/***************************************************************************/

int main( argc, argv)

int argc;
char **argv;

{
  char               Program_Name[256] = "Read_Ulasav";
  float              version = 1.0;
  char               version_date[256] = "24May2002";

  struct ulasav_info header;

  char              *OutputFilePath;
  char              *DataFilePath;
  char              *HeaderText;
  char               Search_String[17];
  char               rowtext[17],columntext[17];

  long int           StressPeriod = 1;
  long int           TimeStep = 1;
  float             *arrayptr[20000], array_value;

  FILE              *DataFilePtr = ( FILE * ) NULL;
  FILE              *OutputFilePtr = ( FILE * ) NULL;

  char               item_name[17];
  long int           i, ii, iii, data_found = 0, n = 0;
  long int           array_size;
  long int           maxlay = 0, layer;
  long int           number_of_records = 0;
  long int           number_of_cells = 0;
  long int           size_of_file, save_pos;

  int                error = -9999, mode = SEEK_CUR;

  long int           seqnum, nlay, nrow, ncol;
  long int           cols, rows, row, rowcolumn, row_multiplier;
  long int           column, column_multiplier;
  long int           record = 0;

/* Disclaimer */
 
  Disclaimer (Program_Name, version, version_date);

/* Parse command line */

  if( argc < 4 || argc > 6 )
    {
     fprintf( stderr,"\n\nUsage: ulasav_txt <Output File Name> <MODFLOW Binary File Name> <Header Text: hed | ddn> {Stress Period} {Time Step}\n\n" );
     exit( 1 );
    }

  OutputFilePath=argv[1];
  DataFilePath=argv[2];
  HeaderText=argv[3];

  if( argc >= 5 )
    StressPeriod=strtol(argv[4],(char **)NULL,10);

  if( argc >= 6 )
    TimeStep=strtol(argv[5],(char **)NULL,10);

/* Check Header String */

  if(strncmp(HeaderText,"hed",3) == 0)
     strncpy(Search_String,"            HEAD\0",17);
  else if(strncmp(HeaderText,"ddn",3) == 0)
     strncpy(Search_String,"        DRAWDOWN\0",17);
  else 
    {
     fprintf(stdout,"\nInvalid text string %s; ",HeaderText);
     fprintf(stdout,"\n  Package          Cell-by-Cell Term      Use");
     fprintf(stdout,"\n  Basx                          head      hed");
     fprintf(stdout,"\n  Basx                      drawdown      ddn");
     fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
     exit (9999);
    }

  fprintf(stdout,"\nSearching file %s for text string %s \n  at time step %d in stress period %d",
          DataFilePath, Search_String, StressPeriod, TimeStep);

/* 
   Open Data File 
*/
 if(( DataFilePtr = fopen( DataFilePath, "rb" )) == NULL )
   {
    fprintf(stdout,"\nCan not open data file %s for reading\n",DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
    return;
   }

 save_pos = ftell (DataFilePtr);      
 fseek (DataFilePtr, 0L, SEEK_END);      
 size_of_file = ftell (DataFilePtr) - 1;      
 fseek (DataFilePtr, save_pos, SEEK_SET);
 i = 0;
 record = 0;

/* 
   Read Data File (end of file or necessary to ensure correct position)
*/

 while (!feof(DataFilePtr))
   {

    /* 
       Read Header Information in Data File 
    */
        
    if(fread((char *) &header,sizeof(struct ulasav_info),1,DataFilePtr) != 1)
      {
       if(ferror(DataFilePtr))
         {
          fprintf(stdout,"\nCould not read header information");
          fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
          exit (9999);
         }
      }
           
    nrow = header.model_rows;
    sprintf(rowtext,"%d",nrow);
    sprintf(rowtext,"%d",strlen(rowtext));
    row_multiplier = 1;
    for(i = 0; i < atoi(rowtext); i++)
      {
	row_multiplier *= 10;
      }
    ncol = header.model_cols;
    sprintf(columntext,"%d",ncol);
    sprintf(columntext,"%d",strlen(columntext));
    column_multiplier = 1;
    for(i = 0; i < atoi(columntext); i++)
      {
	column_multiplier *= 10;
      }
    nlay = header.model_lays;
    for(i = 0; i < strlen(header.current_label); i++)
      {
	if(header.current_label[i] == 'l')
          header.current_label[i] = '\0';
      }
    fprintf(stdout,"\n%20s",header.current_label);
    fprintf(stdout," %6d",header.current_stress_period);
    fprintf(stdout," %6d",header.current_time_step);
    fprintf(stdout," %6d",nrow);
    fprintf(stdout," %7d",ncol);
    fprintf(stdout," %6d",nlay);

    array_size = nrow*ncol;

    /* 
       Check stress period and time step 
    */
    if(header.current_stress_period > StressPeriod)
       break;
    if(header.current_stress_period >= StressPeriod &&
       header.current_time_step > TimeStep)
       break;
         
    /* 
       Right data for right stress period and time step 
    */
    if((strncmp(header.current_label,Search_String,16) == 0) &&
       (StressPeriod == header.current_stress_period) &&
       (TimeStep == header.current_time_step))
      {

       /* 
          Allocate memory for array 
       */
       if(!(arrayptr[nlay] = (float *) malloc(nrow*ncol*sizeof(float *))))
         {
          fprintf(stdout,"\nCould not allocate sufficient memory space for array");
          fprintf(stdout,"\nAbnormal Termination of Ubudsv_Grid_Subroutine\n");
          exit (9999);
         }
                

       data_found = 1;
           
       /* 
          Read array 
       */
       if(fread(arrayptr[nlay],array_size*sizeof(float),1,DataFilePtr) != 1)
         {
          if(feof(DataFilePtr))
             fprintf(stdout,"\nPremature end of file text");
          else
             fprintf(stdout,"\nCould not read array");
          fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
          exit (9999);
         }
           
       /* 
          Layer
       */
       if(nlay > maxlay) {
	 maxlay = nlay;
       }
      }
   
    /* 
       Wrong data 
    */
    else
      {     
       /* 
          Move byte cursor
       */
       error = fseek(DataFilePtr,array_size*sizeof(float),mode);

       /* 
          Check for errors 
       */
       if(error > 0)
         {
          if(feof(DataFilePtr))
             fprintf(stdout,"\nPremature end of file text %d",error);
          else
             fprintf(stdout,"\nCould not read array %d",error);
          fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
          exit (9999);
         }
      }

    /* 
       Reached end of file (necessary to ensure correct position)
    */
    if(ftell(DataFilePtr) >= size_of_file)
       break;
   }
      
/* 
  Close Data File 
*/
 if(fclose(DataFilePtr) != 0)
   {
    fprintf(stdout,"\nUnable to close file %s\n", DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
    exit (9999);
   }

/* 
  Find any data 
*/
 if(data_found > 0)
   {
    fprintf(stdout,"\n\nFound data for %s in file %s for %d layers\n", 
               Search_String,
               DataFilePath,
               maxlay);

    /* 
       Open Output File 
    */
     
    if(( OutputFilePtr = fopen( OutputFilePath, "w+" )) == NULL )
      {
       fprintf(stdout,"\nCan't open output file %s for writing\n",OutputFilePath);
       fprintf(stdout,"\nAbnormal Termination of Ulasav_Program\n");
       exit (9999);
      }
     
    /* 
       Field header
    */
    fprintf(OutputFilePtr,"%s,%s,%s,%s,%s","Seqnum","Layer","Row","Column","Rowcolumn");
   
    for(i = 1; i <= maxlay; i++)
       {
        sprintf(item_name,"%s%d", "LAYER_\0",i);
        
        fprintf(OutputFilePtr,",%s",item_name);
       }

    /* 
       Loop to print cell values array 
    */
    row = 1;
    column = 0;
    seqnum = 0;
        
    for(i = 0; i < nrow*ncol; i++)
       {
        column++;
        if(column > ncol)
          {
           row++;
           column = 1;
          }
          seqnum++;
          rowcolumn = row * column_multiplier + column;
                
          fprintf(OutputFilePtr,"\n%d,%d,%d,%d,%d",
                                seqnum,layer,row,column,rowcolumn);
            
          for(ii = 1; ii <= maxlay; ii++)
             {
	      array_value = *(arrayptr[ii] + seqnum - 1);
              fprintf(OutputFilePtr,",%.3f",array_value);
             }
       }
             
   /* 
      Close Output File 
   */
   if(fclose(OutputFilePtr) != 0)
     fprintf(stdout,"\nUnable to close file %s\n", OutputFilePath);
   }
 else
   {
    fprintf(stdout,"\n\nUnable to find data for %s in file %s\n", 
               Search_String,
               DataFilePath);
    fprintf(stdout,"\nAbnormal Termination of Ulasav_Subroutine\n");
    exit (9999);
   }

}

/***************************************************************************/

void Disclaimer ( Program_Name, version, version_date )

char *Program_Name;
float version;
char *version_date;

/***************************************************************************/
/*
       Name: Disclaimer
    Purpose: Display USGS disclaimer text
*/
/***************************************************************************/
{
      char              fmtbuf[128];
      char              blank = ' ';
      short int         even_length;


  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n************** U.S. Geological Survey preliminary computer program ************");
  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n");
  even_length = 40 - ( short int ) ( strlen(Program_Name) / 2);
  sprintf( fmtbuf, "%%%dc", even_length );
  fprintf(stdout,fmtbuf,blank);
  fprintf(stdout,"%-s",Program_Name);
  fprintf(stdout,"\n                                   version %5.1f",version);
  fprintf(stdout,"\n                                   %14s                 ",version_date);
  fprintf(stdout,"\n*******************************************************************************");
  fprintf(stdout,"\n===============================================================================");
  fprintf(stdout,"\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::");
  fprintf(stdout,"\n**");
  fprintf(stdout,"\n** Disclaimer: ");
  fprintf(stdout,"\n**          Although this program has been used by the U.S. Geological Survey,");
  fprintf(stdout,"\n**           no warranty, expressed or implied, is made by the USGS as to the");
  fprintf(stdout,"\n**           accuracy and functioning of the program and related program");
  fprintf(stdout,"\n**           material nor shall the fact of distribution constitute any such");
  fprintf(stdout,"\n**           warranty, and no responsibility is assumed by the USGS in");
  fprintf(stdout,"\n**           connection therewith.");
  fprintf(stdout,"\n**");
  fprintf(stdout,"\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::");
  fprintf(stdout,"\n===============================================================================");

      fprintf( stdout, "\n" );

}                                           /* end of Disclaimer         */
/*                                            end of Ubudsv.c           */
